"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.TheiaUpdaterImpl = void 0;
/********************************************************************************
 * Copyright (C) 2020 TypeFox, EclipseSource and others.
 *
 * This program and the accompanying materials are made available under the
 * terms of the Eclipse Public License v. 2.0 which is available at
 * http://www.eclipse.org/legal/epl-2.0.
 *
 * This Source Code may also be made available under the following Secondary
 * Licenses when the conditions for such availability set forth in the Eclipse
 * Public License v. 2.0 are satisfied: GNU General Public License, version 2
 * with the GNU Classpath Exception which is available at
 * https://www.gnu.org/software/classpath/license.html.
 *
 * SPDX-License-Identifier: EPL-2.0 OR GPL-2.0 WITH Classpath-exception-2.0
 ********************************************************************************/
const fs = require("fs-extra");
const http = require("http");
const os = require("os");
const path = require("path");
const inversify_1 = require("@theia/core/shared/inversify");
const { autoUpdater } = require('electron-updater');
autoUpdater.logger = require('electron-log');
autoUpdater.logger.transports.file.level = 'info';
let TheiaUpdaterImpl = class TheiaUpdaterImpl {
    constructor() {
        this.clients = [];
        this.initialCheck = true;
        this.reportOnFirstRegistration = false;
        autoUpdater.autoDownload = false;
        autoUpdater.on('update-available', () => {
            const startupCheck = this.initialCheck;
            if (this.initialCheck) {
                this.initialCheck = false;
                if (this.clients.length === 0) {
                    this.reportOnFirstRegistration = true;
                }
            }
            this.clients.forEach(c => c.updateAvailable(true, startupCheck));
        });
        autoUpdater.on('update-not-available', () => {
            if (this.initialCheck) {
                this.initialCheck = false;
                /* do not report that no update is available on start up */
                return;
            }
            this.clients.forEach(c => c.updateAvailable(false, false));
        });
        autoUpdater.on('update-downloaded', () => {
            this.clients.forEach(c => c.notifyReadyToInstall());
        });
        autoUpdater.on('error', (err) => {
            const errorLogPath = autoUpdater.logger.transports.file.getFile().path;
            this.clients.forEach(c => c.reportError({ message: 'An error has occurred while attempting to update.', errorLogPath }));
        });
    }
    checkForUpdates() {
        autoUpdater.checkForUpdates();
    }
    onRestartToUpdateRequested() {
        autoUpdater.quitAndInstall();
    }
    downloadUpdate() {
        autoUpdater.downloadUpdate();
        // record download stat, ignore errors
        fs.mkdtemp(path.join(os.tmpdir(), 'theia-blueprint-updater-'))
            .then(tmpDir => {
            const file = fs.createWriteStream(path.join(tmpDir, 'update'));
            http.get('https://www.eclipse.org/downloads/download.php?file=/theia/update&r=1', response => {
                response.pipe(file);
                file.on('finish', () => {
                    file.close();
                });
            });
        });
    }
    onStart(application) {
        // Called when the contribution is starting. You can use both async and sync code from here.
        this.checkForUpdates();
    }
    onStop(application) {
        // Invoked when the contribution is stopping. You can clean up things here. You are not allowed call async code from here.
    }
    setClient(client) {
        if (client) {
            this.clients.push(client);
            if (this.reportOnFirstRegistration) {
                this.reportOnFirstRegistration = false;
                this.clients.forEach(c => c.updateAvailable(true, true));
            }
        }
    }
    disconnectClient(client) {
        const index = this.clients.indexOf(client);
        if (index !== -1) {
            this.clients.splice(index, 1);
        }
    }
    dispose() {
        this.clients.forEach(this.disconnectClient.bind(this));
    }
};
TheiaUpdaterImpl = __decorate([
    inversify_1.injectable(),
    __metadata("design:paramtypes", [])
], TheiaUpdaterImpl);
exports.TheiaUpdaterImpl = TheiaUpdaterImpl;
//# sourceMappingURL=theia-updater-impl.js.map